@@ -37,7 +37,7 @@
 #include <eel/eel-string.h>
 #include <gtk/gtk.h>
 #include <libgnome/gnome-config.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnome/gnome-url.h>
