@@ -54,7 +54,7 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenushell.h>
 #include <gtk/gtkclipboard.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnomeui/gnome-popup-menu.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -157,15 +157,13 @@ show_iter_for_file (FMTreeView *view, NautilusFile *file, GtkTreeIter *iter)
 	/* check if file is visible in the same root as the currently selected folder is */
 	gtk_tree_view_get_cursor (view->details->tree_widget, &path, NULL);
 	if (path != NULL) {
-		if (gtk_tree_model_get_iter (model, &cur_iter, path)) {
-			if (fm_tree_model_file_get_iter (view->details->child_model,
-							       iter, file, &cur_iter)) {
-				gtk_tree_path_free (path);
-				return TRUE;
-			} else {
-				gtk_tree_path_free (path);
-			}
+		if (gtk_tree_model_get_iter (model, &cur_iter, path) &&
+		    fm_tree_model_file_get_iter (view->details->child_model, iter,
+						 file, &cur_iter)) {
+			gtk_tree_path_free (path);
+			return TRUE;
 		}
+		gtk_tree_path_free (path);
 	}
 	/* check if file is visible at all */
 	if (fm_tree_model_file_get_iter (view->details->child_model,
@@ -1253,6 +1251,7 @@ create_tree (FMTreeView *view)
 	
 	volume_monitor = gnome_vfs_get_volume_monitor ();
 	volumes = gnome_vfs_volume_monitor_get_mounted_volumes (volume_monitor);
+	volumes = g_list_sort (volumes, (GCompareFunc) gnome_vfs_volume_compare);
 	for (l = volumes; l != NULL; l = l->next) {
 		add_root_for_volume (view, l->data);
 		gnome_vfs_volume_unref (l->data);
