@@ -37,7 +37,7 @@
 #include <gtk/gtksizegroup.h>
 
 #include <libgnome/gnome-help.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <glade/glade.h>
 
